'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.INDICES = undefined;
exports.updateStructureIndex = updateStructureIndex;
exports.clearEdgeFromStructureIndex = clearEdgeFromStructureIndex;
exports.clearStructureIndex = clearStructureIndex;

var _utils = require('./utils');

var INDICES = exports.INDICES = new _utils.BasicSet(['structure', 'neighbors']);

/**
 * Structure.
 */

/**
 * Function updating the 'structure' index with the given edge's data.
 *
 * @param {Graph}  graph - Target Graph instance.
 * @param {any}    edge  - Added edge.
 * @param {object} data  - Attached data.
 */
/**
 * Graphology Indexes Functions
 * =============================
 *
 * Bunch of functions used to compute or clear indexes.
 */
function updateStructureIndex(graph, edge, data) {
  var map = graph.map,
      CSet = map ? Set : _utils.BasicSet;

  // Retrieving edge information
  var undirected = data.undirected;
  var source = data.source;
  var target = data.target;

  // Retrieving source & target data

  var sourceData = map ? graph._nodes.get(source) : graph._nodes[source],
      targetData = map ? graph._nodes.get(target) : graph._nodes[target];

  var outKey = undirected ? 'undirectedOut' : 'out',
      inKey = undirected ? 'undirectedIn' : 'in';

  // NOTE: The set of edges is the same for source & target
  var commonSet = new CSet();

  // Handling source
  sourceData[outKey] = sourceData[outKey] || (map ? new Map() : {});

  if (map) {
    if (!sourceData[outKey].has(target)) sourceData[outKey].set(target, commonSet);
    sourceData[outKey].get(target).add(edge);
  } else {
    if (!(target in sourceData[outKey])) sourceData[outKey][target] = commonSet;
    sourceData[outKey][target].add(edge);
  }

  // If selfLoop, we break here
  if (source === target) return;

  // Handling target (we won't add the edge because it was already taken
  // care of with source above)
  targetData[inKey] = targetData[inKey] || (map ? new Map() : {});

  if (map) {
    if (!targetData[inKey].has(source)) targetData[inKey].set(source, commonSet);
  } else {
    if (!(source in targetData[inKey])) targetData[inKey][source] = commonSet;
  }
}

/**
 * Function clearing the 'structure' index data related to the given edge.
 *
 * @param {Graph}  graph - Target Graph instance.
 * @param {any}    edge  - Dropped edge.
 * @param {object} data  - Attached data.
 */
function clearEdgeFromStructureIndex(graph, edge, data) {
  var source = data.source;
  var target = data.target;
  var undirected = data.undirected;
  var map = graph.map;

  // NOTE: since the edge set is the same for source & target, we can only
  // affect source
  var sourceData = map ? graph._nodes.get(source) : graph._nodes[source];

  var outKey = undirected ? 'undirectedOut' : 'out';

  var sourceIndex = sourceData[outKey];

  // NOTE: possible to clear empty sets from memory altogether
  if (map) {
    if (sourceIndex.has(target)) sourceIndex.get(target).delete(edge);
  } else {
    if (target in sourceIndex) sourceIndex[target].delete(edge);
  }
}

/**
 * Function clearing the whole 'structure' index.
 *
 * @param {Graph} graph - Target Graph instance.
 */
function clearStructureIndex(graph) {
  if (graph.map) {
    graph._nodes.forEach(function (data) {

      // Clearing properties
      delete data.in;
      delete data.out;
      delete data.undirectedIn;
      delete data.undirectedOut;
    });
  } else {
    for (var node in graph._nodes) {
      var data = graph._nodes[node];

      // Clearing properties
      delete data.in;
      delete data.out;
      delete data.undirectedIn;
      delete data.undirectedOut;
    }
  }
}