'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = instantiation;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-unused-vars: 0 */
/**
 * Graphology Instantiation Specs
 * ===============================
 *
 * Testing the instantiation of the graph.
 */
var CONSTRUCTORS = ['DirectedGraph', 'UndirectedGraph', 'MultiDirectedGraph', 'MultiUndirectedGraph', 'GraphMap', 'DirectedGraphMap', 'UndirectedGraphMap', 'MultiDirectedGraphMap', 'MultiUndirectedGraphMap'];

var OPTIONS = [{ map: false, multi: false, type: 'directed' }, { map: false, multi: false, type: 'undirected' }, { map: false, multi: true, type: 'directed' }, { map: false, multi: true, type: 'undirected' }, { map: true, multi: false, type: 'mixed' }, { map: true, multi: false, type: 'directed' }, { map: true, multi: false, type: 'undirected' }, { map: true, multi: true, type: 'directed' }, { map: true, multi: true, type: 'undirected' }];

function instantiation(Graph, inmplementation, checkers) {
  var invalid = checkers.invalid;


  return {
    'Hydratation': {

      'it should be possible to hydrate from a serialized graph.': function itShouldBePossibleToHydrateFromASerializedGraph() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Thomas']);
        graph.addEdge('John', 'Thomas');

        var other = new Graph(graph);

        _assert2.default.deepEqual(graph.nodes(), other.nodes());
        _assert2.default.deepEqual(graph.edges(), other.edges());
      },

      'it should be possible to hydrate from a Graph instance.': function itShouldBePossibleToHydrateFromAGraphInstance() {
        var graph = new Graph({
          nodes: [{ key: 'John' }, { key: 'Thomas' }],
          edges: [{ source: 'John', target: 'Thomas' }]
        });

        _assert2.default.deepEqual(graph.nodes(), ['John', 'Thomas']);
        _assert2.default.strictEqual(graph.hasEdge('John', 'Thomas'), true);
      }
    },

    'Options': {

      /**
       * allowSelfLoops
       */
      'allowSelfLoops': {

        'providing a non-boolean value should throw.': function providingANonBooleanValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { allowSelfLoops: 'test' });
          }, invalid());
        }
      },

      /**
       * defaultEdgeAttributes
       */
      'defaultEdgeAttributes': {

        'providing something other than a plain object should throw.': function providingSomethingOtherThanAPlainObjectShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { defaultEdgeAttributes: 'test' });
          }, invalid());
        },

        'it should set default attributes on edges.': function itShouldSetDefaultAttributesOnEdges() {
          var graph = new Graph(null, { defaultEdgeAttributes: { type: 'KNOWS' }, multi: true });

          graph.addNodesFrom(['John', 'Martha']);

          var edge = graph.addEdge('John', 'Martha');

          _assert2.default.deepEqual(graph.getEdgeAttributes(edge), { type: 'KNOWS' });

          edge = graph.addEdge('John', 'Martha', { weight: 3 });

          _assert2.default.deepEqual(graph.getEdgeAttributes(edge), { weight: 3, type: 'KNOWS' });

          edge = graph.addEdge('John', 'Martha', { type: 'LIKES' });

          _assert2.default.deepEqual(graph.getEdgeAttributes(edge), { type: 'LIKES' });
        }
      },

      /**
       * defaultNodeAttributes
       */
      'defaultNodeAttributes': {

        'providing something other than a plain object should throw.': function providingSomethingOtherThanAPlainObjectShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { defaultNodeAttributes: 'test' });
          }, invalid());
        },

        'it should set default attributes on nodes.': function itShouldSetDefaultAttributesOnNodes() {
          var graph = new Graph(null, { defaultNodeAttributes: { eyes: 'blue' } });

          graph.addNode('John');

          _assert2.default.deepEqual(graph.getNodeAttributes('John'), { eyes: 'blue' });

          graph.addNode('Thomas', { age: 23 });

          _assert2.default.deepEqual(graph.getNodeAttributes('Thomas'), { age: 23, eyes: 'blue' });

          graph.addNode('Martha', { eyes: 'green' });

          _assert2.default.deepEqual(graph.getNodeAttributes('Martha'), { eyes: 'green' });
        }
      },

      /**
       * edgeKeyGenerator
       */
      'edgeKeyGenerator': {

        'providing something other than a function should throw.': function providingSomethingOtherThanAFunctionShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { edgeKeyGenerator: 'test' });
          }, invalid());
        }
      },

      /**
       * map
       */
      'map': {

        'providing a non-boolean value should throw.': function providingANonBooleanValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { map: 'test' });
          }, invalid());
        }
      },

      /**
       * multi
       */
      'multi': {

        'providing a non-boolean value should throw.': function providingANonBooleanValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { multi: 'test' });
          }, invalid());
        }
      },

      /**
       * onDuplicateEdge
       */
      'onDuplicateEdge': {

        'providing a non-function truthy value should throw.': function providingANonFunctionTruthyValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { onDuplicateEdge: 'test' });
          }, invalid());
        },

        'the callback should fire if the user add the same edge twice.': function theCallbackShouldFireIfTheUserAddTheSameEdgeTwice() {

          var callback = (0, _helpers.spy)(function (g, data) {
            (0, _assert2.default)(g instanceof Graph);
            _assert2.default.deepEqual(data.attributes, { type: 'KNOWS' });
            _assert2.default.strictEqual(data.source, 'John');
            _assert2.default.strictEqual(data.target, 'Martha');
            _assert2.default.strictEqual(data.undirected, false);

            g.updateEdgeAttribute(data.source, data.target, 'weight', function (x) {
              return (x || 1) + 1;
            });
          });

          var graph = new Graph(null, { onDuplicateEdge: callback });

          graph.addNodesFrom(['John', 'Martha']);

          var edge = graph.addEdge('John', 'Martha', { type: 'KNOWS' });

          _assert2.default.doesNotThrow(function () {
            graph.addEdge('John', 'Martha', { type: 'KNOWS' });
          });

          _assert2.default.strictEqual(graph.size, 1);
          _assert2.default.deepEqual(graph.getEdgeAttributes('John', 'Martha'), { type: 'KNOWS', weight: 2 });

          (0, _assert2.default)(callback.called);
          (0, _assert2.default)(callback.times, 1);
        }
      },

      /**
       * onDuplicateNode
       */
      'onDuplicateNode': {

        'providing a non-function truthy value should throw.': function providingANonFunctionTruthyValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { onDuplicateNode: 'test' });
          }, invalid());
        },

        'the callback should fire if the user add the same node twice.': function theCallbackShouldFireIfTheUserAddTheSameNodeTwice() {
          var callback = (0, _helpers.spy)(function (g, data) {
            (0, _assert2.default)(g instanceof Graph);
            _assert2.default.strictEqual(data.key, 'John');
            _assert2.default.deepEqual(data.attributes, { age: 34 });

            g.updateNodeAttribute('John', 'occurrences', function (x) {
              return (x || 1) + 1;
            });
          });

          var graph = new Graph(null, { onDuplicateNode: callback });

          graph.addNode('John', { age: 34 });

          _assert2.default.doesNotThrow(function () {
            graph.addNode('John', { age: 34 });
          });

          _assert2.default.strictEqual(graph.order, 1);
          _assert2.default.deepEqual(graph.getNodeAttributes('John'), { age: 34, occurrences: 2 });

          (0, _assert2.default)(callback.called);
          (0, _assert2.default)(callback.times, 1);
        }
      },

      /**
       * type
       */
      'type': {

        'providing an invalid type should throw.': function providingAnInvalidTypeShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { type: 'test' });
          }, invalid());
        }
      }
    },

    'Constructors': {

      'all alternative constructors should be available.': function allAlternativeConstructorsShouldBeAvailable() {
        CONSTRUCTORS.forEach(function (name) {
          return (0, _assert2.default)(name in inmplementation);
        });
      },

      'alternative constructors should have the correct options.': function alternativeConstructorsShouldHaveTheCorrectOptions() {
        CONSTRUCTORS.forEach(function (name, index) {
          var graph = new inmplementation[name]();

          var _OPTIONS$index = OPTIONS[index];
          var map = _OPTIONS$index.map;
          var multi = _OPTIONS$index.multi;
          var type = _OPTIONS$index.type;


          _assert2.default.strictEqual(graph.map, map);
          _assert2.default.strictEqual(graph.multi, multi);
          _assert2.default.strictEqual(graph.type, type);
        });
      }
    }
  };
}